<?php
namespace OCA\Dematpayslip\Db;

use JsonSerializable;

use OCP\AppFramework\Db\Entity;

class DematpayslipArchive extends Entity implements JsonSerializable {

    /**
     * Const status
     */
    const STATUS_DELETED       = -1;
    const STATUS_INITIALIZED   = 0;
    const STATUS_CREATED       = 1;
    const STATUS_TITLED        = 2;
    const STATUS_FILE_SENT     = 3;
    const STATUS_SAE_SENT      = 4;
    const STATUS_METADATA_SENT = 5;
    const STATUS_ORIENTATION   = 6;

    /**
     * Const step status
     */
    const STEP_STATUS_INIT     = 0;
    const STEP_STATUS_PROGRESS = 1;
    const STEP_STATUS_ARCHIVED = 2;

    protected $idFile;
    protected $idUser;
    protected $filePath;
    protected $dematpayslipIdUser;
    protected $pastellIdDocument;
    protected $siret;
    protected $matricule;
    protected $year;
    protected $month;
    protected $lastname;
    protected $firstname;
    protected $startDate;
    protected $endDate;
    protected $editionDate;
    protected $status;
    protected $statusTime;
    protected $pastellUrl;
    protected $pastellLastAction;
    protected $importTime;
    protected $checksum;

    public function jsonSerialize() {
        return [
            'id'                 => $this->id,
            'idFile'             => $this->idFile,
            'idUser'             => $this->idUser,
            'filePath'           => $this->filePath,
            'dematpayslipIdUser' => $this->dematpayslipIdUser,
            'pastellIdDocument'  => $this->pastellIdDocument,
            'siret'              => $this->siret,
            'matricule'          => $this->matricule,
            'year'               => $this->year,
            'month'              => $this->month,
            'lastname'           => $this->lastname,
            'firstname'          => $this->firstname,
            'startDate'          => $this->startDate,
            'endDate'            => $this->endDate,
            'editionDate'        => $this->editionDate,
            'status'             => $this->status,
            'status_time'        => $this->statusTime,
            'pastellUrl'         => $this->pastellUrl,
            'pastellLastAction'  => $this->pastellLastAction,
            'importTime'         => $this->importTime,
            'checksum'           => $this->checksum,
        ];
    }

    /**
     * Get status list
     *
     * @param   int     $stepStatus     Step status
     * @return  array   Status list
     */
    public static function getStepStatusListFrom($stepStatus) {
        $stepStatusList = [];

        if ($stepStatus == self::STEP_STATUS_INIT) {
            $stepStatusList[] =  self::STATUS_INITIALIZED;
        } else if ($stepStatus == self::STEP_STATUS_PROGRESS) {
            $stepStatusList[] = self::STATUS_CREATED;
            $stepStatusList[] = self::STATUS_TITLED;
            $stepStatusList[] = self::STATUS_FILE_SENT;
            $stepStatusList[] = self::STATUS_SAE_SENT;
            $stepStatusList[] = self::STATUS_METADATA_SENT;
        } else if ($stepStatus == self::STEP_STATUS_ARCHIVED) {
            $stepStatusList[] = self::STATUS_ORIENTATION;
        }

        return $stepStatusList;
    }

    /**
     * Show status label
     *
     * @return  string  Status label
     */
    public function showStatusLabel() {
        $pastellLastAction = $this->getPastellLastAction();

        $statusLabelList = array(
            self::STATUS_DELETED       => 'Supprimé',
            self::STATUS_INITIALIZED   => 'Aucun',
            self::STATUS_CREATED       => 'Créé',
            self::STATUS_TITLED        => 'Titre ajouté',
            self::STATUS_FILE_SENT     => 'Fichier envoyé',
            self::STATUS_SAE_SENT      => 'Envoyé au SAE',
            self::STATUS_METADATA_SENT => 'Méta-données transmises',
            self::STATUS_ORIENTATION   => 'Orienté',
        );

        $statusLabel  = $statusLabelList[$this->getStatus()];
        if (!empty($pastellLastAction)) {
            $statusLabel .= ' ';
            if ($pastellLastAction == 'ar-recu-sae') {
                $statusLabel .= '<span style="color: #777000;">';
            } else if ($pastellLastAction == 'accepter-sae') {
                $statusLabel .= '<span style="color: #007700;">';
            } else if ($pastellLastAction == 'rejet-sae') {
                $statusLabel .= '<span style="color: #770000;">';
            } else {
                $statusLabel .= '<span>';
            }
            $statusLabel .=  '[' . $pastellLastAction . ']';
            $statusLabel .= '</span>';
        }

        return $statusLabel;
    }

    /**
     * Determine if it have already been archived (can't modify)
     *
     * @return  bool
     */
    public function isArchived() {
        $isArchived = false;

        if (!empty($this->getPastellIdDocument())) {
            if (in_array($this->getStatus(), self::getStepStatusListFrom(self::STEP_STATUS_ARCHIVED))) {
                $isArchived = true;
            }
        }

        return $isArchived;
    }
}
